
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved.

/****************************************************************************
						Microsoft RPC
           
                      InOut Example

    FILE:       inoutc.c

    USAGE:      inoutc   -n network_address
                         -p protocol_sequence
                         -e endpoint
                         -a server principal name
                         -o options
                         -1 short_value_1
                         -2 short_value_2
                         -3 float_value_3

    PURPOSE:    Client side of RPC distributed application inout

    FUNCTIONS:  main() - binds to server and calls remote procedure

    COMMENTS:   This distributed application demonstrates in, out
                parameters.

****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "inout.h"    // header file generated by MIDL compiler
#include "spn.h"

#define PURPOSE \
"This Microsoft RPC Version 2.0 sample program demonstrates\n\
use of the [in], [out], and [in, out] attributes. For more\n\
information about attributes and RPC API functions, see the\n\
Microsoft RPC programming guide and reference.\n\n"

#define DESCRIPTION \
"One [in], one [out], and one [in,out] parameter are defined\n\
for the function InOutProc(). This program displays the values\n\
of these parameters before and after the remote procedure call.\n\n"

void Usage(char * pszProgramName)
{
    fprintf_s(stderr, "%s", PURPOSE);
    fprintf_s(stderr, "Usage:  %s\n", pszProgramName);
    fprintf_s(stderr, " -p protocol_sequence\n");
    fprintf_s(stderr, " -n network_address\n");
    fprintf_s(stderr, " -e endpoint\n");
    fprintf_s(stderr, " -a server principal name\n");	
    fprintf_s(stderr, " -o options\n");
    fprintf_s(stderr, " -1 parameter_1\n");
    fprintf_s(stderr, " -2 parameter_2\n");
    fprintf_s(stderr, " -3 parameter_3\n");
    exit(1);
}

void __cdecl main(int argc, char **argv)
{
    RPC_STATUS status;
    unsigned char * pszUuid             = NULL;
    unsigned char * pszProtocolSequence = "ncacn_ip_tcp";
    unsigned char * pszNetworkAddress   = NULL;
    unsigned char * pszEndpoint         = "8765";
    unsigned char * pszSpn              = NULL;	
    unsigned char * pszOptions          = NULL;
    unsigned char * pszStringBinding    = NULL;
	RPC_SECURITY_QOS SecQos;
    short   s1 = 257;
    short   s2 = 631;
    float   f3 = (float) 0.406;
    int i;

    /* allow the user to override settings with command line switches */
    for (i = 1; i < argc; i++) {
        if ((*argv[i] == '-') || (*argv[i] == '/')) {
            switch (tolower(*(argv[i]+1))) {
            case 'p':  // protocol sequence
                pszProtocolSequence = argv[++i];
                break;
            case 'n':  // network address
                pszNetworkAddress = argv[++i];
                break;
            case 'e':
                pszEndpoint = argv[++i];
                break;
            case 'a': 
                pszSpn = argv[++i];
                break;
            case 'o':
                pszOptions = argv[++i];
                break;
            case '1':
                s1 = (short) atoi(argv[++i]);
                break;
            case '2':
                s2 = (short) atoi(argv[++i]);
                break;
            case '3':
                f3 = (float) atof(argv[++i]);
                break;
            case 'h':
            case '?':
            default:
                Usage(argv[0]);
            }
        }
        else
            Usage(argv[0]);
    }

    printf_s("%s", DESCRIPTION);

    /* Use a convenience function to concatenate the elements of  */
    /* the string binding into the proper sequence.               */
    status = RpcStringBindingCompose(pszUuid,
                                     pszProtocolSequence,
                                     pszNetworkAddress,
                                     pszEndpoint,
                                     pszOptions,
                                     &pszStringBinding);
    printf_s("RpcStringBindingCompose returned 0x%x\n", status);
    printf_s("pszStringBinding = %s\n", pszStringBinding);
    if (status) {
        exit(status);
    }

    /* Set the binding handle that will be used to bind to the server. */
    status = RpcBindingFromStringBinding(pszStringBinding,
                                         &inout_IfHandle);
    printf_s("RpcBindingFromStringBinding returned 0x%x\n", status);
    if (status) {
        exit(status);
    }
	
    /* User did not specify spn, construct one. */
    if (pszSpn == NULL) {
        MakeSpn(&pszSpn);
    }

    /* Set the quality of service on the binding handle */
    SecQos.Version = RPC_C_SECURITY_QOS_VERSION_1;
    SecQos.Capabilities = RPC_C_QOS_CAPABILITIES_MUTUAL_AUTH;
    SecQos.IdentityTracking = RPC_C_QOS_IDENTITY_DYNAMIC;
    SecQos.ImpersonationType = RPC_C_IMP_LEVEL_IDENTIFY;
	
    /* Set the security provider on binding handle */
    status = RpcBindingSetAuthInfoEx(inout_IfHandle,
                                     pszSpn,
                                     RPC_C_AUTHN_LEVEL_PKT_PRIVACY,
                                     RPC_C_AUTHN_GSS_NEGOTIATE,
                                     NULL,
                                     RPC_C_AUTHZ_NONE,
                                     &SecQos);
	
    printf_s("RpcBindingSetAuthInfoEx returned 0x%x\n", status);
    if (status) {
        exit(status);
    }	
		
    printf_s("Calling the remote procedure 'InOutProc'\n");
    printf_s("  parameters = %d %d %0.3f\n", s1, s2, f3);

    RpcTryExcept {
        InOutProc(inout_IfHandle,s1, &s2, &f3);  // call the remote procedure

        printf_s("Returning from the remote procedure 'InOutProc'\n");
        printf_s("  parameters = %d %d %0.3f\n", s1, s2, f3);

        Shutdown(inout_IfHandle);
    }
    RpcExcept(( ( (RpcExceptionCode() != STATUS_ACCESS_VIOLATION) &&
                   (RpcExceptionCode() != STATUS_DATATYPE_MISALIGNMENT) &&
                   (RpcExceptionCode() != STATUS_PRIVILEGED_INSTRUCTION) &&
                   (RpcExceptionCode() != STATUS_BREAKPOINT) &&
                   (RpcExceptionCode() != STATUS_STACK_OVERFLOW) &&
                   (RpcExceptionCode() != STATUS_IN_PAGE_ERROR) &&
                   (RpcExceptionCode() != STATUS_GUARD_PAGE_VIOLATION)
                    )
                    ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH )) {
        printf_s("Runtime reported exception %ld\n", RpcExceptionCode() );

	}
    RpcEndExcept

    /* The call to the remote procedure is complete. */
    /* Free the string and binding handle.           */
    status = RpcBindingFree(&inout_IfHandle);
    printf_s("RpcBindingFree returned 0x%x\n", status);
    if (status) {
        exit(status);
    }

    status = RpcStringFree(&pszStringBinding);
    printf_s("RpcStringFree returned 0x%x\n", status);
    if (status) {
        exit(status);
    }

    exit(0);

}  // end main()


/*********************************************************************/
/*                 MIDL allocate and free                            */
/*********************************************************************/

void __RPC_FAR * __RPC_USER midl_user_allocate(size_t len)
{
    return(malloc(len));
}

void __RPC_USER midl_user_free(void __RPC_FAR * ptr)
{
    free(ptr);
}

/* end file inoutc.c */
